/// @addtogroup dynsys
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file BasicC2Taylor.hpp
///
/// @author Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file constitutes a part of the CAPD library, 
// distributed under the terms of the GNU General Public License.
// Consult  http://capd.wsb-nlu.edu.pl/ for details. 

#ifndef _CAPD_DYNSYS_BASICC2TAYLOR_HPP_ 
#define _CAPD_DYNSYS_BASICC2TAYLOR_HPP_ 

#include <string>
#include <stdexcept>

#include "capd/map/C2Map.hpp"
#include "capd/dynsys/BasicC2Taylor.h"

namespace capd{
namespace dynsys{

//###########################################################//

template<typename MapType>
BasicC2Taylor<MapType>::BasicC2Taylor(
      MapType& vectorField,
      int order,
      const ScalarType& step
  ) : BasicTaylor<MapType>(vectorField,order,step)
{
  m_c2MatrixCoeff = new (m_dimension) C2CoeffType[m_order+2];
}


//###########################################################//


template<typename MapType>
void BasicC2Taylor<MapType>::setOrder(int newOrder)
{
  if(newOrder==m_order)
    return;
  BasicTaylor<MapType>::setOrder(newOrder);
  delete [] m_c2MatrixCoeff;
  m_c2MatrixCoeff = new (m_dimension) C2CoeffType[m_order+2];
}

//###########################################################//

template<typename MapType>
typename BasicC2Taylor<MapType>::VectorType BasicC2Taylor<MapType>::operator()(
      const VectorType& x,
      MatrixType& jac,
      C2CoeffType& hessian
   )
{
  C2CoeffType H(m_dimension);
  VectorType result(m_dimension);
  H.clear();
  hessian.clear();
  jac.clear();
  result.clear();

  computeC2Coeff(x, MatrixType::Identity(m_dimension), H, m_order-1);

  ScalarType stepPower(1.);
  for(int p=0;p<=m_order;++p)
  {
    typename C2CoeffType::iterator i1 = hessian.begin(),
                                    i2 = hessian.end(),
                                    j1 = m_c2MatrixCoeff[p].begin();
    while(i1!=i2)
    {
      *i1 += *j1 * stepPower;
      ++i1;
      ++j1;
    }
    jac += m_matrixCoeff[p] * stepPower;
    result += m_coeff[p] * stepPower;
    stepPower *= m_step;
  }
  return result;
}


// ####################################################################


template<typename MapType>
void BasicC2Taylor<MapType>::computeC2Coeff(
      const VectorType& x,
      const MatrixType& V,
      const C2CoeffType& H, int order
  ) const
{
  if(order<0)
  {
    throw std::out_of_range("C2Taylor error: negative index of Taylor coefficient in C2Taylor::computeC2Coeff");
  }

  int p,i,j,k,s,r,w,c;

// first we compute ceofficients for x
  m_coeff[0]=x;
  for(r=0;r<=order;++r)
    m_coeff[r+1] = (*m_vField)(m_coeff[r],r)/(r+1);


  computeMatrixCoeff(order,V);

  m_c2MatrixCoeff[0]=H;
  for(i=1;i<=order+1;++i)
    m_c2MatrixCoeff[i].clear();
  C2CoeffType* temp = m_vField->computeSecondDerivatives(m_coeff,order);

// we compute coefficients of H_{ijc} up to order 'ile+1'

  for(p=0;p<=order;++p)
  {
    for(i=0;i<m_dimension;++i)
    {
      for(j=0;j<m_dimension;++j)
      {
        for(c=0;c<=j;++c)
        {
// -------------
          for(k=0;k<m_dimension;++k)
          {
            for(r=0;r<m_dimension;++r)
            {
              for(w=0;w<=p;++w)
              {
                ScalarType S1(0.);
                for(s=0;s<=w;++s)
                  S1 += temp[s](i,k,r) * m_matrixCoeff[w-s][r][c];

                m_c2MatrixCoeff[p+1](i,j,c) += S1 * m_matrixCoeff[p-w][k][j];
              }  // end of w-for
            }  // end of r-for
          }  // end of k-for

          for(k=0;k<m_dimension;++k)
            for(s=0;s<=p;++s)
                m_c2MatrixCoeff[p+1](i,j,c) +=  m_F[s][i][k] * m_c2MatrixCoeff[p-s](k,j,c);

          m_c2MatrixCoeff[p+1](i,j,c) = m_c2MatrixCoeff[p+1](i,j,c)/(p+1);
// -------------
        } // end of c-for
      } // end of j-for
    } //end of i-for
  }

  delete []temp;
}

}} // namespace capd::dynsys

#endif // _CAPD_DYNSYS_BASICC2TAYLOR_HPP_ 

/// @}
