/// @addtogroup dynsys
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file Taylor.hpp
///
/// @author Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file constitutes a part of the CAPD library, 
// distributed under the terms of the GNU General Public License.
// Consult  http://capd.wsb-nlu.edu.pl/ for details. 

/* Author: Daniel Wilczak 2001-2004 */

#ifndef _CAPD_DYNSYS_TAYLOR_HPP_ 
#define _CAPD_DYNSYS_TAYLOR_HPP_ 

#include <sstream>
#include <string>
#include <stdexcept>

#include "capd/capd/factrial.h"

#include "capd/dynsys/Taylor.h"
#include "capd/dynsys/TaylorException.h"

#include "capd/dynsys/DynSys.hpp"
#include "capd/dynsys/BasicTaylor.hpp"
#include "capd/dynsys/enclosure.hpp"

namespace capd{
namespace dynsys{

//###########################################################//

template<typename MapType>
Taylor<MapType>::Taylor(MapType& vectorField, int a_order, const ScalarType& Step)
  : BasicTaylor<MapType>(vectorField,a_order,Step)
{}


//###########################################################//

template<typename MapType>
typename Taylor<MapType>::MatrixType Taylor<MapType>::JacPhi(const VectorType &iv) const
{
  // first we compute ceofficients for x
  int r;
  m_coeff[0] = iv;
  for(r=0;r<m_order;++r)
    m_coeff[r+1] = (*m_vField)(m_coeff[r],r)/(r+1);

  // this function compute coefficients up to index 'order'
  computeMatrixCoeff(m_order-1,MatrixType::Identity(m_dimension));

  // the sumation of the Taylor series
  MatrixType result = m_matrixCoeff[m_order];
  for(r=m_order-1;r>=0;--r)
    result = result*m_step + m_matrixCoeff[r];

  return result;
}


//###########################################################//


template<typename MapType>
typename Taylor<MapType>::VectorType Taylor<MapType>::Remainder(const VectorType &iv) const
{
  // we compute an enclosure for \varphi([0,timestep],iv)
  int ok;
  m_coeff[0] = enclosure(iv,&ok);

  // the computation of Taylor coefficients up to an order <order+1> on <enc>
  for(int i=0;i<=m_order;++i)
    m_coeff[i+1] = (*m_vField)(m_coeff[i],i)/(i+1);

  return m_coeff[m_order+1]*power(m_step,m_order+1);
}

//###########################################################//


template<typename MapType>
typename Taylor<MapType>::VectorType Taylor<MapType>::enclosure(const VectorType &x) const
// the function finds an enclosure for \varphi([0,step],x)
{
  return capd::dynsys::enclosure(*m_vField, x, m_step);
}

//###########################################################//
template<typename MapType>
typename Taylor<MapType>::VectorType Taylor<MapType>::enclosure(const VectorType &x, int *found) const
// the function finds an enclosure for \varphi([0,step],x)
{
  return enclosure(x);
}


//###########################################################//


template<typename MapType>
typename Taylor<MapType>::MatrixType Taylor<MapType>::jacEnclosure(const VectorType &enc, const NormType &the_norm) const
// the function finds enclosure for Jacobian matrix (variational part)
// source- "C^1-Lohner algorithm" by P. Zgliczynski
{
  return capd::dynsys::jacEnclosure(*m_vField,m_step,enc,the_norm);
}


//###########################################################//


template<typename MapType>
void Taylor<MapType>::JacRemainder(
      const VectorType &vecEnc,
      const MatrixType &jacEnc,
      VectorType &Rem,
      MatrixType &jacRem
  ) const
// the remainder for varaiational part is computed
   // vecEnc - enclosure for \varphi([0,step],x)
   // jacEnc -  enclosure for V([0,step],x)
{
  m_coeff[0] = vecEnc;
  int r;
  for(r=0;r<=m_order;++r)
    m_coeff[r+1] = (*m_vField)(m_coeff[r],r)/(r+1);

  computeMatrixCoeff(m_order,jacEnc);

  ScalarType fac = power(m_step,m_order+1);
  Rem = m_coeff[m_order+1] * fac;
  jacRem = m_matrixCoeff[m_order+1] * fac;
}


//###########################################################//

}} //namespace capd::dynsys

#endif // _CAPD_DYNSYS_TAYLOR_HPP_ 

/// @}
