/// @addtogroup poincare
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file TimeMapTSC.h
///
/// @author Tomasz Kapela
///
/// Time Map class with Time Step Control (TSC)
///
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#ifndef _CAPD_POINCARE_TIME_MAP_TSC_H_
#define _CAPD_POINCARE_TIME_MAP_TSC_H_

#include <string>

namespace capd{
namespace poincare{


// default values of the parameters

#define  __FIRST_TIME_STEP   (1./1024.)
#define  __MIN_TIME_STEP   (1./1048576.)
 // 2^-20
#define  __DEFAULT_ACCURACY  1.e-21
#define  __NUMBER_OF_TERMS   2

template<typename DS>
class TimeMapTSC
{
public:
  typedef typename DS::MapType MapType;
  typedef typename DS::FunctionType FunctionType;
  typedef typename DS::MatrixType MatrixType;
  typedef typename DS::VectorType VectorType;
  typedef typename DS::ScalarType ScalarType;
 //  typedef capd::dynset::C0set<MatrixType> C0set;
 //  typedef capd::dynset::C1set<MatrixType> C1set;
 //  typedef capd::dynset::C2set<MatrixType> C2set;

  TimeMapTSC(DS &dynamicalSystem,
         double minStep = __MIN_TIME_STEP,
         double accuracy = __DEFAULT_ACCURACY,
         int nTerms = __NUMBER_OF_TERMS,
         double firstStep = __FIRST_TIME_STEP);

  VectorType operator()(ScalarType time, VectorType v);

  VectorType operator()(ScalarType time, VectorType v, MatrixType &derivative);

  template<typename SetType>
  VectorType operator()(ScalarType time, SetType &theSet );

  const MapType& getField() const;
  MapType& getField();
  int getOrder() const;
  const ScalarType& getStep() const;

  void setOrder(int newOrder);
  void setStep(const ScalarType& newStep);
  void setMinStep( double minStep);
  void setAccuracy( double accuracy);
  void setNumberOfTerms( int nTerms);
  void setFirstTimeStep( double firstStep);

protected:

  ScalarType setNextTimeStep();
  ScalarType setNextTimeStep(VectorType v);

// the fields of the class

  DS& dynamicalSystem; // a template dynamical system (Taylor or C2Taylor)
  int m_dim;
  int numberOfTerms;  // number of highest terms to consider in next step computation
  double epsilon;     // desired precision of computations
  double minTimeStep; // minimal time step we want to use
  double firstTimeStep; // time step used for simulation to compute first time step
}; // end of template TimeMapTSC

// -----------------------  inline definitions -------------------------------

template <typename DS>
inline TimeMapTSC<DS>::TimeMapTSC(
           DS &ds,
           double minStep,
           double accuracy,
           int nTerms,
           double firstStep
      ) :  dynamicalSystem(ds)

{
  m_dim = ds.getField().getDimension();
  firstTimeStep = firstStep;
  minTimeStep = minStep;
  epsilon = accuracy ;
  numberOfTerms = nTerms;
}

template<typename DS>
inline const typename TimeMapTSC<DS>::MapType& TimeMapTSC<DS>::getField() const
{
  return dynamicalSystem.getField();
}

template<typename DS>
inline typename TimeMapTSC<DS>::MapType& TimeMapTSC<DS>::getField() 
{
  return dynamicalSystem.getField();
}

template<typename DS>
inline int TimeMapTSC<DS>::getOrder() const
{
  return dynamicalSystem.getOrder();
}

template<typename DS>
inline const typename TimeMapTSC<DS>::ScalarType& TimeMapTSC<DS>::getStep() const
{
  return dynamicalSystem.getStep();
}

template<typename DS>
inline void TimeMapTSC<DS>::setOrder(int newOrder)
{
  dynamicalSystem.setOrder(newOrder);
}

template<typename DS>
inline void TimeMapTSC<DS>::setStep(const ScalarType& newStep)
{
  dynamicalSystem.setStep(newStep);
}

template<typename DS>
inline void TimeMapTSC<DS>::setMinStep( double minStep)
{
  minTimeStep = minStep;
}

template<typename DS>
inline void TimeMapTSC<DS>::setAccuracy( double accuracy)
{
  epsilon = accuracy;
}

template<typename DS>
inline void TimeMapTSC<DS>::setNumberOfTerms( int nTerms)
{
  numberOfTerms = nTerms;
}

template<typename DS>
inline void TimeMapTSC<DS>::setFirstTimeStep( double firstStep)
{
  firstTimeStep = firstStep;
}

}} // namespace capd::poincare

#endif // _CAPD_POINCARE_TIME_MAP_TSC_H_

/// @}
