/// @addtogroup poincare
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file timeStep.hpp
///
/// @author Tomasz Kapela, Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#ifndef _CAPD_POINCARE_TIME_STEP_HPP_
#define _CAPD_POINCARE_TIME_STEP_HPP_

#include "capd/interval/DoubleInterval.h"
#include "capd/dynsys/enclosure.hpp"

namespace capd{
namespace poincare{


// calculates optimal time step - it changes step of dynamicalSystem
// Do not use this before first step (we need taylor coefficient to be already computed)
template<typename DS>
typename DS::ScalarType computeNextTimeStep(DS& dynamicalSystem, const typename DS::VectorType& x)
{
  typedef typename DS::ScalarType ScalarType;
  typedef typename ScalarType::BoundType Real;
  typedef typename DS::VectorType VectorType;

  int numberOfTerms = 2;
  Real epsilon(1.e-21);
  Real minTimeStep(1./1048576.);
  
  int order = dynamicalSystem.getOrder();
  Real optStep(1.e100) , step = dynamicalSystem.getStep().leftBound();

  for(int i = order; (i>order - numberOfTerms)&&(i>0); i--)
  {
    Real coeffNorm = dynamicalSystem.getCoeffNorm(i).rightBound();
    step = exp(log(epsilon / coeffNorm)/ i );
  }

  optStep = capd::max(step/2, minTimeStep);
  while(optStep>=minTimeStep)
  {
    try{
      capd::dynsys::enclosure(dynamicalSystem.getField(),x,optStep);
      // if succeed then break
      break;
    }catch(...)
    {
      optStep *= 0.8;
    }
  }
  return optStep;
}


}} // namespace capd::poincare


#endif // #define _CAPD_POINCARE_TIME_STEP_HPP_

/// @}
