#include "nBodyProblem.h"
#include "subVector.h"
#include "textfile.h"
#include <exception>

// Dimension of the phase space
#define  DIM_SP (4*nBodies-4)
#define  DIM_NORMAL (4*nBodies)

// Dimension of full space in parallel shooting
#define  DIM_FULL  (DIM_SP * nDiv)

// Dimension of reduced space
#define  DIM_RED (DIM_FULL-2)


IntervalMatrix NBodyProblem::operator[] (IntervalVector X)
{

    scon << "\n\n computation of dF(X)";

    timeMap.setOrder(Xorder);
    timeMap.setStep(Xstep);

    IntervalVector X_e = E * X;

    IntervalMatrix dP(DIM_FULL, DIM_FULL);

    setZC = Shift * calculatePM(X_e, dP);        //  Compute Poicare Map and its derivative

    IntervalMatrix dF = (R * Shift * dP * E - IntervalMatrix::Identity(DIM_RED));

    return  dF;

}

IntervalVector NBodyProblem::operator() (IntervalVector X)
{
    scon << "\n\n computation of F(x0)";

    timeMap.setOrder(order);
    timeMap.setStep(step);

    IntervalVector X_e = E * X;

    IntervalMatrix dP(DIM_FULL, DIM_FULL);

    IntervalVector PX = calculatePM(X_e, dP);        //  Compute Poicare Map and its derivative

    IntervalVector F = (R * Shift * PX  - X);

    return  F;
}

IntervalVector NBodyProblem::operator() (IntervalVector X, IntervalMatrix &dF)
{

    scon << "\n\n computation of F(x0) and dF(x0)";

    timeMap.setOrder(order);
    timeMap.setStep(step);

    IntervalVector X_e = E * X;

    IntervalMatrix dP(DIM_FULL, DIM_FULL);

    IntervalVector PX = calculatePM(X_e, dP);        //  Compute Poicare Map and its derivative

    dF = (R * Shift * dP * E - IntervalMatrix::Identity(DIM_RED));

    IntervalVector F = (R * Shift * PX  - X);

    return  F;

}


/////////////////////////////////////////////////////////////////////////////////
///  Calculates Poincare Map and its derivative 
////////////////////////////////////////////////////////////////////////////////

IntervalVector NBodyProblem::calculatePM(IntervalVector X, IntervalMatrix &dP)
{

  IntervalVector x0(DIM_SP),
                 tempResult(DIM_SP);
  IntervalVector result(DIM_FULL);
  IntervalMatrix derivative(DIM_FULL, DIM_FULL),
                 dTemp(DIM_SP, DIM_SP);
  LNorm norm;
  derivative.clear();
  for(int d=0; d < nDiv; d++)
  {
    for(int i=0; i<DIM_SP; i++)
        x0[i]=X[DIM_SP*d+i];
	  try
    {
      C1rect2 S(x0,norm);
      tempResult = timeMap(time, S);
      dTemp = IntervalMatrix(S);

      copyToSubvector(tempResult, result, d*DIM_SP, (d+1)*DIM_SP);
      copyToSubmatrix(dTemp, derivative, d*DIM_SP, (d+1)*DIM_SP);
    }
    catch(std::exception &e)
    {
      scon << e.what();
      throw std::runtime_error(" Error calculating Poincare Map. " );
    }

  }
  dP = derivative;
  return result;
}

IntervalMatrix NBodyProblem::calculateDiff(const IntervalVector &X)
{
     timeMap.setOrder(Xorder);
     timeMap.setStep(Xstep);

     IntervalVector X_e = E * X;

     IntervalMatrix dP(DIM_FULL, DIM_FULL);

     calculatePM (X_e, dP);
     return dP;
}

bool NBodyProblem::uniquenessTest()
{
   IntervalVector C(2), c0(2);
   IntervalVector setZC_1(DIM_SP);

   // We take only entries from the first configuration
   for(int i=0; i<DIM_SP; ++i)
      setZC_1[i] = setZC[i];

   C[0] = setZC[1];  C[1] = setZC[2];
   c0[0] = 0.0;      c0[1] = 0.0;

   Map J = reducedFirstIntegrals2D(nBodies);

   IntervalMatrix dJ_dx = J[setZC_1];

   IntervalMatrix dJ_dc(2,2);

   for(int i=0; i<2; i++)
      for(int j=1; j<3; j++)
         dJ_dc[i][j-1] = dJ_dx[i][j];

   Interval det = dJ_dc[0][0]*dJ_dc[1][1] - dJ_dc[0][1]*dJ_dc[1][0];

   sout << "\n\n Uniqueness test: "
        << "\n\n set C " << C
        << "\n\n point c0" << c0
        << "\n\n dJ/dc(Z,C) = \n" << dJ_dc
        << "\n\n det(dJ/dc(Z,C)) = " << det;

   if((det.contains(0.0))||(!subset(c0,C)))
      return false;
   else
      return true;

}

// Generate (N-1) bodies shift
IntervalMatrix NBodyProblem::generateBodiesShift()
{
   IntervalMatrix S(DIM_SP, DIM_SP);
   S.clear();
   for( int i=0; i< DIM_SP; ++i)
      S[DIM_SP - 4 + i%4][i] = Interval(-1.0);
   for( int i=0; i < DIM_SP-4; ++i)
      S[i][i+4] = Interval(1.0);
   return S;
}

IntervalMatrix NBodyProblem::generateBodiesShift_1()
{
   IntervalMatrix S(DIM_SP, DIM_SP);
   S.clear();
   for( int i=0; i< DIM_SP; ++i)
      S[i%4][i] = Interval(-1.0);
   for( int i=4; i < DIM_SP; ++i)
      S[i][i-4] = Interval(1.0);
   return S;
}

IntervalMatrix NBodyProblem::monodromyMatrix(IntervalVector X)
{
  timeMap.setOrder(Xorder);
  timeMap.setStep(Xstep);

  IntervalVector X_e = E * X;

  IntervalMatrix dP(DIM_FULL, DIM_FULL);

  calculatePM(X_e, dP);        //  Compute Poicare Map and its derivative

  IntervalMatrix dP2(DIM_SP, DIM_SP);
  copyFromSubmatrix(dP, dP2, 0, DIM_SP);

  IntervalMatrix A(DIM_SP, DIM_SP);

  for(int i=1; i<nDiv; ++i)
  {
     copyFromSubmatrix(dP, A, DIM_SP * i, DIM_SP*(i+1));
     dP2 = A * dP2 ;
  }

  IntervalMatrix dP3 = dP2;
  IntervalMatrix BodiesShift = generateBodiesShift();
  IntervalMatrix BodiesShift_1 = generateBodiesShift_1();
  IntervalMatrix dP4 = dP2 * BodiesShift_1;
  
  for(int i=1; i<nBodies; ++i)
  {
     dP3 = dP4 * dP3;
  }
  IntervalMatrix S = BodiesShift;

  for(int i = 2; i <nBodies;  ++i)
  {
    S = S*BodiesShift;
  }

  dP3 = S * dP3;

  return  dP3;

}

