
#ifndef _NBODYPROBLEM2_H_
#define _NBODYPROBLEM2_H_

#include "typedefs.h"
#include "nbody.h"
#include "textfile.h"
#include "mapping.h"

class NBodyProblem2 : public capd::newton::Mapping<IntervalMatrix>
{
  public:
    typedef IntervalMatrix::ScalarType IntervalType;
    typedef IntervalMatrix::RowVectorType VectorType;
    typedef IntervalMatrix MatrixType;
    typedef IntervalType::BoundType ScalarType;

    NBodyProblem2(int A_nBodies, int A_nDiv, Interval A_time,
                  int A_orderBegin, int A_orderEnd, int A_orderStep,
                  ScalarType A_stepBegin, ScalarType A_stepEnd, ScalarType A_stepFactor,
                  int A_XorderBegin, int A_XorderEnd, int A_XorderStep,
                  ScalarType A_XstepBegin, ScalarType A_XstepEnd, ScalarType A_XstepFactor
                 );

    virtual ~NBodyProblem2() {}

    IntervalMatrix operator[] (IntervalVector X);  // Derivative of the Map
    IntervalVector operator() (IntervalVector X);  // Value of the Map
    IntervalVector operator() (IntervalVector X, IntervalMatrix &dF);  // Value of the Map and its derivative dF

    int getDimension() const;
    bool uniquenessTest();


    int nBodies;
    int nDiv;

   protected:
    Interval time;
    int orderBegin, orderEnd, orderStep,
        XorderBegin, XorderEnd, XorderStep;
    ScalarType stepBegin, stepEnd, stepFactor,
           XstepBegin, XstepEnd, XstepFactor;
    Map vectorField;
    Taylor taylor;
    TimeMap timeMap;
    IntervalMatrix  E,
                    Shift,
                    R;
    IntervalVector setZC;        // Set Z x C for uniqueness test

    IntervalVector calculatePM(IntervalVector X, IntervalMatrix &dP,
                               int orderBegin, int orderEnd, int orderStep,
                               ScalarType stepBegin, ScalarType stepEnd, ScalarType stepFactor);
    IntervalMatrix generateE();
    IntervalMatrix generateShift();
    IntervalMatrix generateR();
};



inline NBodyProblem2::NBodyProblem2(int A_nBodies, int A_nDiv, Interval A_time,
                                    int A_orderBegin, int A_orderEnd, int A_orderStep,
                                    ScalarType A_stepBegin, ScalarType A_stepEnd, ScalarType A_stepFactor,
                                    int A_XorderBegin, int A_XorderEnd, int A_XorderStep,
                                    ScalarType A_XstepBegin, ScalarType A_XstepEnd, ScalarType A_XstepFactor
                                    )
                           : nBodies(A_nBodies),
                             nDiv(A_nDiv),
                             time(A_time),
                             orderBegin(A_orderBegin),  orderEnd(A_orderEnd),  orderStep(A_orderStep),
                             XorderBegin(A_XorderBegin), XorderEnd(A_XorderEnd), XorderStep(A_XorderStep),
                             stepBegin(A_stepBegin), stepEnd(A_stepEnd), stepFactor(A_stepFactor),
                             XstepBegin(A_XstepBegin), XstepEnd(A_XstepEnd), XstepFactor(A_XstepFactor),
                             vectorField(reducedNBodyEquation2D(A_nBodies)),
                             taylor(vectorField,A_orderBegin, A_stepBegin), 
                             timeMap(taylor)
{
   E = generateE();
   Shift = generateShift();
   R = generateR();
}



inline int NBodyProblem2::getDimension() const
{
   return ((nBodies-1)*4*nDiv-2);
}

// Generates matrix of expansion function
// simply, this function adds two coordinates set to zero
//   (second and third coordinate)
//

inline IntervalMatrix NBodyProblem2::generateE()
{
   IntervalMatrix E((nBodies-1)*4*nDiv,(nBodies-1)*4*nDiv-2);
   E.clear();
   E[0][0]=Interval(1.);
   for(int i=1; i<(nBodies-1)*4*nDiv-2; ++i)
      E[i+2][i]=Interval(1.);
   return E;
}


inline IntervalMatrix NBodyProblem2::generateR()
{
   IntervalMatrix R((nBodies-1)*4*nDiv-2,(nBodies-1)*4*nDiv);
   R.clear();
   R[0][0]=Interval(1.);
   for(int i=1; i<(nBodies-1)*4*nDiv-2; i++)
      R[i][i+2]=Interval(1.);
   return R;
}

inline IntervalMatrix NBodyProblem2::generateShift()
{
   IntervalMatrix S((nBodies-1)*4*nDiv,(nBodies-1)*4*nDiv);
   S.clear();
   int tupleLength = (nBodies-1)*4;
   for(int i=(nDiv-1)*tupleLength, j=0; i< nDiv*tupleLength; i++, j=(j+1)%4)
     S[j][i] = Interval(-1.);

   for(int i=(nDiv-1)*tupleLength, j=4; i< nDiv*tupleLength-4; i++, j++)
     S[j][i] = Interval(1.);

   for(int i=0; i< (nDiv-1)*tupleLength; i++)
     S[tupleLength+i][i] = Interval(1.);

   return S;
}

#endif
