/////////////////////////////////////////////////////////////////////////////
//   Package:    Choreographies in the N-Body Problem
/// @addtogroup NBodyProblem
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file nBodyProblem.cpp
///
/// Definition of the class nBodyProblem 
///
/// @author Tomasz Kapela
/////////////////////////////////////////////////////////////////////////////
// Copyright (C) Tomasz Kapela 2006. 


#include "nBodyProblem.h"
#include "subVector.h"
#include "textfile.h"
#include <exception>


// Dimension of the phase space
#define  DIM_SP  (4*nBodies-4)
#define  DIM_NORMAL  (4*nBodies)

// Dimension of full space in parallel shooting
#define DIM_FULL  (DIM_SP * nDiv)

// Dimension of reduced space
#define DIM_RED  (DIM_FULL-2)


IntervalMatrix NBodyProblem::operator[] (IntervalVector X)
{
    timeMap.setOrder(Xorder);
    timeMap.setStep(Xstep);

    IntervalVector X_e = E * X;

    IntervalMatrix dP(DIM_FULL, DIM_FULL);

    setZC = Shift * calculatePM(X_e, dP);        //  Compute Poicare Map and its derivative

    IntervalMatrix dF = (R * Shift * dP * E - IntervalMatrix::Identity(DIM_RED));

    return  dF;

}

IntervalVector NBodyProblem::operator() (IntervalVector X)
{
    timeMap.setOrder(order);
    timeMap.setStep(step);

    IntervalVector X_e = E * X;

    IntervalMatrix dP(DIM_FULL, DIM_FULL);

    IntervalVector PX = calculatePM(X, dP);        //  Compute Poicare Map and its derivative

    IntervalVector F = (R * Shift * PX  - X);

    return  F;
}

IntervalVector NBodyProblem::operator() (IntervalVector X, IntervalMatrix &dF)
{

    timeMap.setOrder(order);
    timeMap.setStep(step);

    IntervalVector X_e = E * X;

    IntervalMatrix dP(DIM_FULL, DIM_FULL);

    IntervalVector PX = calculatePM(X_e, dP);        //  Compute Poicare Map and its derivative

    dF = (R * Shift * dP * E - IntervalMatrix::Identity(DIM_RED));

    IntervalVector F = (R * Shift * PX  - X);

    return  F;

}

/////////////////////////////////////////////////////////////////////////////////
//   NBodyProblem::calculatePM
///
///  Calculates Poincare Map and its derivative 
///
///  @param[in] X    set of initial configurations
///  @param[out]   dP   derivative of Poicare Map on set X 
///
///  @returns    value of Poincare Map computed on set X 
////////////////////////////////////////////////////////////////////////////////
IntervalVector NBodyProblem::calculatePM(IntervalVector X, IntervalMatrix &dP)
{

   IntervalVector x0(DIM_SP),
                  tempResult(DIM_SP);
   IntervalVector result(DIM_FULL);
   IntervalMatrix derivative(DIM_FULL, DIM_FULL),
                  dTemp(DIM_SP, DIM_SP);
   LNorm norm;
   derivative.clear();

   if(nDiv>1)
      scon << "\n part : " ;

   for(int d=0; d < nDiv; d++)
   {
      if(nDiv>1)
         scon << (d+1) << " ";

      for(int i=0; i<DIM_SP; i++)
         x0[i]=X[DIM_SP*d+i];
      try
      {
         C1rect2 S(x0,norm);

         tempResult = timeMap(time, S);
         dTemp = IntervalMatrix(S);

         copyToSubvector(tempResult, result, d*DIM_SP, (d+1)*DIM_SP);
         copyToSubmatrix(dTemp, derivative, d*DIM_SP, (d+1)*DIM_SP);
      }
      catch(std::exception &e)
      {
        scon << e.what();
        throw std::runtime_error(" Error calculating Poincare Map. " );
      }

   }
   dP = derivative;
   return result;
}


/////////////////////////////////////////////////////////////////////////////////
//   NBodyProblem::calculateDiff
///
///  Calculates derivative of the Poincare Map 
///
///  @param   X    set of initial configurations
///
///  @returns     derivative of Poicare Map on set X 
////////////////////////////////////////////////////////////////////////////////
IntervalMatrix NBodyProblem::calculateDiff(const IntervalVector &X)
{
     timeMap.setOrder(Xorder);
     timeMap.setStep(Xstep);

     IntervalVector X_e = E * X;

     IntervalMatrix dP(DIM_FULL, DIM_FULL);

     calculatePM (X_e, dP);
     return dP;
}

bool NBodyProblem::uniquenessTest()
{
   IntervalVector C(2), c0(2);
   IntervalVector setZC_1(DIM_SP);

   // We take only entries from the first configuration
   for(int i=0; i<DIM_SP; ++i)
      setZC_1[i] = setZC[i];

   C[0] = setZC[1];              C[1] = setZC[2];
   c0[0] = ScalarType(0.0);      c0[1] = ScalarType(0.0);

   Map J = reducedFirstIntegrals2D(nBodies);

   IntervalMatrix dJ_dx = J[setZC_1];

   IntervalMatrix dJ_dc(2,2);

   for(int i=0; i<2; i++)
      for(int j=1; j<3; j++)
         dJ_dc[i][j-1] = dJ_dx[i][j];

   Interval det = dJ_dc[0][0]*dJ_dc[1][1] - dJ_dc[0][1]*dJ_dc[1][0];

   sout << "\n\n dJ/dc(Z,C) = \n" << dJ_dc
        << "\n\n det(dJ/dc(Z,C)) = " << det;

   if((det.contains(0.0))||(!subset(c0,C)))
      return false;
   else
      return true;

}


//////////////////////////////////////////////////////////////////////////////
//   NBodyProblem::generateE
///
///  Generates matrix of a linear expansion map.
///  Simply, this function adds two coordinates set to zero
///    (second and third coordinate).
///
///   @return matrix of a linear expansion map
//////////////////////////////////////////////////////////////////////////////
IntervalMatrix NBodyProblem::generateE()
{
   IntervalMatrix E((nBodies-1)*4*nDiv,(nBodies-1)*4*nDiv-2);
   E.clear();
   E[0][0]=Interval(1.);
   for(int i=1; i<(nBodies-1)*4*nDiv-2; ++i)
      E[i+2][i]=Interval(1.);
   return E;
}

//////////////////////////////////////////////////////////////////////////////
//   NBodyProblem::generateR
///
///  Generates matrix of a linear reductions map.
///  Simply, this function removes second and third coordinate.
///
///   @return matrix of a linear reduction map
//////////////////////////////////////////////////////////////////////////////
IntervalMatrix NBodyProblem::generateR()
{
   IntervalMatrix R((nBodies-1)*4*nDiv-2,(nBodies-1)*4*nDiv);
   R.clear();
   R[0][0]=Interval(1.);
   for(int i=1; i<(nBodies-1)*4*nDiv-2; i++)
      R[i][i+2]=Interval(1.);
   return R;
}

//////////////////////////////////////////////////////////////////////////////
//   NBodyProblem::generateShift
///
///  Generates matrix of all nDiv configurations shift in the 'full space'.
///
///   @return matrix of a shift map
//////////////////////////////////////////////////////////////////////////////
IntervalMatrix NBodyProblem::generateShift()
{
   IntervalMatrix S(DIM_FULL, DIM_FULL);
   S.clear();
   int tupleLength = (nBodies-1)*4;
   for(int i=(nDiv-1)*tupleLength, j=0; i< nDiv*tupleLength; i++, j=(j+1)%4)
     S[j][i] = Interval(-1.);

   for(int i=(nDiv-1)*tupleLength, j=4; i< nDiv*tupleLength-4; i++, j++)
     S[j][i] = Interval(1.);

   for(int i=0; i< (nDiv-1)*tupleLength; i++)
     S[tupleLength+i][i] = Interval(1.);

   return S;
}

//////////////////////////////////////////////////////////////////////////////
//   NBodyProblem::generateBodiesShift
///
///  Generates matrix of single configuration shift (N-1 bodies).
///  This function is used for monodromy matrix generation.
///
///  @return matrix of a shift map
//////////////////////////////////////////////////////////////////////////////
IntervalMatrix NBodyProblem::generateBodiesShift()
{
   IntervalMatrix S(DIM_SP, DIM_SP);
   S.clear();
   for( int i=0; i< DIM_SP; ++i)
      S[DIM_SP - 4 + i%4][i] = Interval(-1.0);
   for( int i=0; i < DIM_SP-4; ++i)
      S[i][i+4] = Interval(1.0);
   return S;
}

//////////////////////////////////////////////////////////////////////////////
//   NBodyProblem::generateBodiesShift_1
///
///  Generates matrix of single configuration backshift (N-1 bodies).
///  This function is used for monodromy matrix generation.
///
///  @return matrix of a backshift map
//////////////////////////////////////////////////////////////////////////////
IntervalMatrix NBodyProblem::generateBodiesShift_1()
{
   IntervalMatrix S(DIM_SP, DIM_SP);
   S.clear();
   for( int i=0; i< DIM_SP; ++i)
      S[i%4][i] = Interval(-1.0);
   for( int i=4; i < DIM_SP; ++i)
      S[i][i-4] = Interval(1.0);
   return S;
}

//////////////////////////////////////////////////////////////////////////////
//   NBodyProblem::monodromyMatrix
///
///  Computes monodromy matrix for choreography.  
///
///  This function computes monodromy matrix \f$ \frac{\partial\varphi(T,X)}{\partial x} \f$,
///  where \f$ \varphi \f$ is flow of the N-body equation, 
///        \f$ T \f$ period of the orbit, 
///        \f$ X \f$ set of initial conditions. <br>
///  Flow is integrated only for \f$ T/(N*nDiv) \f$ and then monodromy matrix 
///  is composed using shift of the bodies. 
///  
///  @param  X  Set of initial conditions      
///
///  @return monodromy matrix computed on set X
//////////////////////////////////////////////////////////////////////////////
IntervalMatrix NBodyProblem::monodromyMatrix(const IntervalVector& X)
{
  timeMap.setOrder(Xorder);
  timeMap.setStep(Xstep);
  IntervalVector X_e = E * X;
  IntervalMatrix dP(DIM_FULL, DIM_FULL);  // dP = \f$ \frac{\partial\varphi(T/(N\times nDiv),X)}{\partial x} \f$,

  calculatePM(X_e, dP);        //  Compute Poicare Map and its derivative 
                               //  after time = T/(N*nDiv)

  IntervalMatrix dP2(DIM_SP, DIM_SP);          // dP2 = derivative after time T/N      
  copyFromSubmatrix(dP, dP2, 0, DIM_SP);       
  IntervalMatrix A(DIM_SP, DIM_SP);            // We compose dP2 using parts of trajectory 
  for(int i=1; i<nDiv; ++i)                    // from parallel shooting
  {
     copyFromSubmatrix(dP, A, DIM_SP * i, DIM_SP*(i+1));
     dP2 = A * dP2 ;
  }

  IntervalMatrix dP3 = dP2;                    // dP3 = derivative after time T      
 //  dP3 =(BodiesShift^(N-1))*dP2*(BodiesShift_1^(N-1))*...*(BodiesShift*dP2*BodiesShift_1)*dP2;
 //      but some factors cancels
   
  IntervalMatrix BodiesShift = generateBodiesShift();
  IntervalMatrix BodiesShift_1 = generateBodiesShift_1();
  IntervalMatrix dP4 = dP2 * BodiesShift_1;  
  for(int i=1; i<nBodies; ++i)       
  {                                 // We compose dP3 using parts of trajectory 
     dP3 = dP4 * dP3;               // by particles interchanging 
  }

  IntervalMatrix S = BodiesShift;
  for(int i = 2; i <nBodies;  ++i)
    S = S*BodiesShift;
  
  dP3 = S * dP3;
  
  return  dP3;

}

/// @}


