

#include <sstream>
#include <iomanip>
#include <stdexcept>


//#include "dimen.h"
#include "typedefs.h"
#include "nbody.h"
#include "proof.h"
#include "textfile.h"
#include "toText.h"

namespace capd{


//*****************************************************************************/
/*   PROOF  - Rigorous computation   -  we use Krawczyk metod                 */
//   to prove existence of zero of funtion F(x)
/*****************************************************************************/

int KrawczykProof (FloatVector &x,  double size,  Mapping &F)
{

   int dim = F.dimension();

   IntervalVector X(dim),           // initial conditions
                  x0(dim);

   for(int i=0; i< dim; i++)
   {
       X[i]=Interval(x[i]-size, x[i]+size);
       x0[i] = Interval(x[i]);
   }
   return KrawczykProof(x0, X, F);

}

std::string KrawczykInfo(int iter,   IntervalVector x0,   IntervalVector X,
                         IntervalVector F_x0,  IntervalMatrix dF_X,  IntervalVector K)
{
   std::ostringstream str;
   str << "\n Number of iterations : " << iter
       << "\n\n" << toText("F(x0)" , F_x0,  16, false)
       << "\n\n" << toText("dF(X)" , dF_X,  16, false)
       << "\n\n" << toText("X" , X,  16, true)
       << "\n\n" << toText("K" , K,  16, true);

   return str.str();

}


int KrawczykProof (IntervalVector x0, IntervalVector X, Mapping &F)
{

 int dim = F.dimension();

 IntervalVector F_x0(dim),        //  function F computed at point x0
                K(dim);           // value of Krawczyk operator

 IntervalMatrix C(dim, dim),        // Matrix C for Krawczyk metod
                dF_X(dim, dim);      //  derivative of F computed on set X


 int exit_code = -1;
 int comp_F_x0 = 1 ;           // flag : 1 = computation of F(x_0) is needed
 int iter;

 for(iter=1; iter<8; iter++)  // iteration of taking intersection and Krawczyk metod
 {
    scon <<" PROOF:  RIGOROUS COMPUTATION\n iteration nr "<< iter<<"\n";

    // Computation for a point x0
    if(comp_F_x0)
    {
       comp_F_x0 =0;

       IntervalMatrix dF_x0;

       F_x0 = F(x0, dF_x0);

       // We take as matrix C for Krawczyk operator an inverse of Jacobi matrix at point x0
       C = capd::vectalg::midMatrix( capd::matrixAlgorithms::gaussInverseMatrix(midMatrix(dF_x0)));

       // We check if C is invertible. If it fails, an excepction will be thrown.
       capd::matrixAlgorithms::gaussInverseMatrix(C);

    }

    // computation for the set X
    dF_X = F[X];

    /*  KRAWCZYK OPERATOR   */
    K = x0 - C * F_x0 +(IntervalMatrix::Identity(dim) - C * dF_X)*(X-x0);   //Krawczyk

    if( subsetInterior(K, X))
    {
        // K is subset of int X  - there exists exactly one zero of F in X
        exit_code = 1;
        break;
    }
    else
    {
       if(subset(X, K))
       {
           // K contains X - try to change parameters
         exit_code = -2;
         break;
       }
       else
       {
          try
          {
            X=intersection(X, K);
            // X is partialy contained in K - we try to take intersection of X and K as new set X
            if( !subset(x0, X))
            {
                x0=midVector(X);
                comp_F_x0=1;
            }
          }
          catch(std::runtime_error &e)
          {
             //Intersection of X and K is empty. There is no zeroes in set X";
             exit_code = 0;
             break;
          }
       }
    }
 }

 sout << KrawczykInfo(iter, x0, X, F_x0, dF_X, K);

 return exit_code;
}



IntervalVector KrawczykOperator (IntervalVector x0,  IntervalVector X, Mapping &F)
{

  int dim = F.dimension();

  IntervalVector F_x0(dim);        //  function F computed at point x0

  IntervalMatrix C(dim, dim),        // Matrix C for Krawczyk metod
                dF_X(dim, dim);      //  derivative of F computed on set X

  IntervalMatrix dF_x0;

  F_x0 = F(x0, dF_x0);

  // We take as matrix C for Krawczyk operator an inverse of the Jacobi matrix at the point x0
  C = midMatrix(capd::matrixAlgorithms::inverseMatrix(midMatrix(dF_x0)));

  // We check if C is invertible. If it fails, an excepction will be thrown.
  capd::matrixAlgorithms::inverseMatrix(C);

  // Derivative of F on set X
  dF_X = F[X];

  /*  KRAWCZYK OPERATOR   */

  IntervalVector K = x0 - C * F_x0 +(IntervalMatrix::Identity(dim) - C * dF_X)*(X-x0);   //Krawczyk

  return K;
}



/*********************************************************************************************/

} // end namespace capd


